/*
 * This file is part of GFMerge. Copyright (c) 2007 Genome
 * Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.psu.gfmerge;

import java.util.*;
import java.io.*;
import org.biojava.bio.*;
import org.biojava.bio.seq.*;
import org.biojava.bio.symbol.*;
import org.biojava.bio.seq.impl.*;
import org.biojava.bio.seq.io.*;

import uk.ac.sanger.psu.gfmerge.datacomp.*;
import uk.ac.sanger.psu.gfmerge.analysis.tools.*;
import uk.ac.sanger.psu.gfmerge.util.io.*;
import uk.ac.sanger.psu.gfmerge.util.getopt.*;
import uk.ac.sanger.psu.gfmerge.util.biotools.*;
import uk.ac.sanger.psu.gfmerge.util.algotools.*;
import uk.ac.sanger.psu.gfmerge.util.debug.*;
import uk.ac.sanger.psu.gfmerge.analysis.*;
import uk.ac.sanger.psu.gfmerge.util.PredAccuracy;

/** class which contains the logic of the program GFMerge
 *
 * @author Sebastian R. Spiegler
 * @version 1.0
 */

public class GFMerge /*extends Debug */ {

  ///////////////////////////////////////
  // GFMerge - static variables

  /** SimpleArg object which contains all command line parameters
   *
   * @see uk.ac.sanger.psu.gfmerge.util.getopt.SimpleArg
   */
  public static SimpleArg allArguments;

  /** Main method of GFMerge which creates the GFMerge_Analysis object
   *
   * @param args command line arguments
   */
  public static void main(String[] args) {

    ///////////////////////////////////////
    // main - variables

    ///////////////////////////////////////
    // main - logic

    long startTime = System.currentTimeMillis();

    // get all arguments from command line into a SimpleOpt object
    allArguments = new SimpleArg(args);
    //debug 101 - print command line arguments
    Debug.debug_101(allArguments.getDebug(), allArguments);

    long analysisStartTime = System.currentTimeMillis();
    // Create GFMerge_Analysis object
    GFMerge_Analysis gfmAnalysis = new GFMerge_Analysis(allArguments);

    // get merged feature table (result of analysis)
    ArrayList mergedFeatureTable = gfmAnalysis.getMergedFeatureTable();

    long analysisEndTime = System.currentTimeMillis();

    // SAVE result of analysis to
    GFMergeOutput.saveFeat2File(
        allArguments.getOutputFilename() + ".tab",
        mergedFeatureTable,
        (Prediction) gfmAnalysis.getPredictions().get(0));

    // SAVE removed genemodels (whole analysis) to output file
    if (allArguments.doFileOfRemovedGms()) {
      String filename =
          allArguments.getOutputFilename() + ".removed" + ".tab";

      GFMergeOutput.saveRemovedFeat2File(
          filename,
          gfmAnalysis.getArrListOfGMRegions_org(),
          mergedFeatureTable,
          (Prediction) gfmAnalysis.getPredictions().get(0));
    }

    long endTime = System.currentTimeMillis();

    // output of the processing time (not important)
    System.out.println(Calendar.DATE);
    System.out.println(allArguments.getPredictedTrainSet());
    System.out.println("Prediction Length: " +
                       ( (Prediction) gfmAnalysis.getPredictions().get(0)).
                       getSequence().length());
    System.out.println("Program start time (ms)   : " + startTime);
    System.out.println("Program analysis time (ms): " + analysisStartTime);
    System.out.println("Program analysis time (ms): " + analysisEndTime);
    System.out.println("Program end time (ms)     : " + endTime);
    System.out.println("Analysis time (ms)        : " +
                       (analysisEndTime - analysisStartTime));
    System.out.println("Processing time (ms)      : " + (endTime - startTime));
    System.out.println("---");

    // comparing the annotation to the merged prediction
    // retrieving the acp value of the merged prediction
    // only works if the sequence of the annotation is the same as the sequence of the merged prediction

    // get sequence object of annotation
    SequenceIterator seqIt_org = GFMergeInput.getSeq(allArguments.
        getAnnotatedTrainSet());
    Sequence seq_org = null;
    try {
      seq_org = BioTools.seqs2Seq(seqIt_org);
    }
    catch (BioException ex) {}

    // get sequence object of merged prediction
    SequenceIterator seqIt_comp = GFMergeInput.getSeq(allArguments.
        getOutputFilename() + ".tab");
    Sequence seq_comp = null;
    try {
      seq_comp = BioTools.seqs2Seq(seqIt_comp);
    }
    catch (BioException ex) {}

    // create acp object in order to calculate the acp value/accuracy of
    // the merged prediction
    // ONLY WORKS IF ANNOTATION IS ON THE SAME SEQUENCE AS MERGED PREDICTION !!!
    if(seq_org.length() == seq_comp.length())
    {
      PredAccuracy acp = new PredAccuracy(seq_org, seq_comp);
      System.out.println("ACP of merged features: " + acp.getACP());
    }

    // main - logic end
    ///////////////////////////////////////

  } //  END OF MAIN

}