/*
 * This file is part of GFMerge. Copyright (c) 2007 Genome
 * Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.psu.gfmerge.analysis.simfeature_analysis;

import java.util.*;
import uk.ac.sanger.psu.gfmerge.util.getopt.*;
import uk.ac.sanger.psu.gfmerge.analysis.tools.*;
import uk.ac.sanger.psu.gfmerge.analysis.simfeature_analysis.tools.*;
import uk.ac.sanger.psu.gfmerge.analysis.tools.*;

/** Analysis class which calculates the high scoring path in a clustered sequence using the average conditional probability
 *
 * @author Sebastian R. Spiegler
 * @see uk.ac.sanger.psu.gfmerge.util.PredAccuracy
 * @version 1.0
 */

public class AvgCondProbAnalysis extends BasicFeatureAnalysis
{

  ///////////////////////////////////////
  // constructor
  /** constructor of AvgCondProbAnalysis objects
   *
   * @param _arrListOfGMRegions ArrayList of genemodel regions
   * @param _allArguments object which contains all command line
   *     parameters
   */
  public AvgCondProbAnalysis(ArrayList _arrListOfGMRegions,
                             SimpleArg _allArguments) {
    super(_arrListOfGMRegions, _allArguments);
  }


  /** method which returns the high scoring path of genemodels on a sequence using the average conditional probability as scoring system
   * <p>
   * If one genemodel overlaps another, the genemodel with the higher overlap is taken.
   * For further details consult the documentation.
   *
   * @param _arrListOfGMRegionsToAnalyze ArrayList of genemodel regions
   * @return ArrayList of regions with high scoring genemodels
   * @see uk.ac.sanger.psu.gfmerge.datacomp.GeneModel
   * @see uk.ac.sanger.psu.gfmerge.datacomp.GFMergeRegion
   */
  public ArrayList getHighScoringGmRegArr(ArrayList _arrListOfGMRegionsToAnalyze)
  {
    ArrayList highScoringGmRegArr   = new ArrayList();
    int debug                       = this.getAllArguments().getDebug();

    // HashMap of gms pointing to their acp value
    HashMap gmHashOfBestAcp =
        AvgCondProbAnalysisTools.getAcpScoredGmHash(debug, _arrListOfGMRegionsToAnalyze);


    // save scores in biojava features ("note" tag)
    SavScoreInBioFeat.savScore(this.getArrListOfGMRegions(),
                               "AvgCondProbAnalysis",
                               gmHashOfBestAcp);


    // HashMap of high scoring path gms
    // AVERAGE score used for high score path
    boolean isAverage = true;
    HashMap highScorePathGms =
        HighScoringPath.getHighScoringPath(gmHashOfBestAcp,
                                           _arrListOfGMRegionsToAnalyze,
                                           isAverage);

    // remove low scoring gms from gm region array
    highScoringGmRegArr =
        HighScoringPath.removeLowScoreGmsFromRegArr(_arrListOfGMRegionsToAnalyze,
        highScorePathGms);

    return highScoringGmRegArr;
  }
}
