/*
 * This file is part of GFMerge. Copyright (c) 2007 Genome
 * Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.psu.gfmerge.analysis.simfeature_analysis;

import java.util.*;
import uk.ac.sanger.psu.gfmerge.util.getopt.*;
import uk.ac.sanger.psu.gfmerge.util.debug.*;
import uk.ac.sanger.psu.gfmerge.analysis.tools.*;
import uk.ac.sanger.psu.gfmerge.analysis.simfeature_analysis.tools.*;

/** Analysis class which calculates the high scoring path in a clustered sequence using a cDNA splice sites scoring system
 *
 * @author Sebastian R. Spiegler
 * @version 1.0
 */
public class CDnaSpliceSiteAnalysis extends FeatureAnalysis
{

  ///////////////////////////////////////
  // constructor
  /** constructor for CDnaSpliceSiteAnalysis objects
   *
   * @param _arrListOfGMRegions ArrayList of genemodel regions
   * @param _arrListOfCDnaRegions ArrayList of cDNA regions
   * @param _allArguments SimpleArg object which contains all command
   *     line arguments
   */
  public CDnaSpliceSiteAnalysis(ArrayList _arrListOfGMRegions,
                                ArrayList _arrListOfCDnaRegions,
                                SimpleArg _allArguments)
  {
    super(_arrListOfGMRegions,
          _arrListOfCDnaRegions,
          _allArguments);
  }


  ///////////////////////////////////////
  // methods

  /** method which returns the high scoring path of genemodels on a sequence using a cDNA splice site scoring system
   * <p>
   * Genemodel splice sites are scored with +1 when they match a cDNA splice site, with -1 when they overlap a cDNA
   * but do not match and 0 when they do not overlap a cDNA. The total score is calculated as the sum of all splice
   * site scores.
   * For further details consult the documentation.
   *
   * @param _arrListOfGMRegionsToAnalyze ArrayList of genemodel regions
   * @return ArrayList of regions with high scoring genemodels
   */
  public ArrayList getHighScoringGmRegArr(ArrayList _arrListOfGMRegionsToAnalyze)
  {
    ArrayList highScoringGmsArr     = new ArrayList();
    ArrayList _arrListOfCDnaRegions = this.getArrListOfFeatRegions();
    int debug                       = this.getAllArguments().getDebug();

    // get scores for gm splice sites
    boolean considerStrand = this.getAllArguments().doConsiderCDnaStrand();
    int fuzzyCDnaBoundaries = this.getAllArguments().getFuzzyBoundariesValue();

    HashMap cDnaScoredHash =
        CDnaSpliceSiteAnalysisTools.getCDnaSpliceSiteScoredGmHash(debug,
                                                          fuzzyCDnaBoundaries,
                                                          considerStrand,
                                                          _arrListOfGMRegionsToAnalyze,
                                                          _arrListOfCDnaRegions);
    // save scores in biojava features ("note" tag)
    SavScoreInBioFeat.savScore(this.getArrListOfGMRegions(),
                               "CDnaSpliceSite",
                               cDnaScoredHash);


    // pick high scoring path
    // (highScoreGmsHash not clustered into regions!!!)

    // NO average score used for high score path
    boolean isAverage = false;
    HashMap highScoreGmsHash
        = HighScoringPath.getHighScoringPath(cDnaScoredHash,
                                             _arrListOfGMRegionsToAnalyze,
                                             isAverage);
       //debug 204 - hash of high scoring path
       Debug.debug_204(debug, "CDnaSpliceSite - hash of high scoring path",
                       highScoreGmsHash);


    // remove all gms with are not on high scoring path
    // remove low scoring gms
    highScoringGmsArr
        = HighScoringPath.removeLowScoreGmsFromRegArr(_arrListOfGMRegionsToAnalyze,
                                                      highScoreGmsHash);
      //debug 205 - cleaned array of gm regions without lowscoring gms
      Debug.debug_205(debug, highScoringGmsArr);


    return highScoringGmsArr;
  }
}
