/*
 * This file is part of GFMerge. Copyright (c) 2003 Genome
 * Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.psu.gfmerge.analysis.simfeature_analysis.tools;

import java.util.*;
import org.biojava.bio.symbol.*;

import uk.ac.sanger.psu.gfmerge.datacomp.*;
import uk.ac.sanger.psu.gfmerge.util.debug.*;

/** class which contains basic computing tools which are used by derived classes
 *
 * @author Sebastian R. Spiegler
 * @version 1.0
 */
public class BasicFeatureAnalysisTools {

  /** method which gets hash of gms pointing to ArrayList of overlapping features
   * (returns only overlapped genemodels !!!)
   *
   * @param debug debugging mode
   * @param considerStrand switch for strand consideration
   * @param _arrListOfGMRegions ArrayList of genemodel regions
   * @param _arrListOfFeatRegions ArrayList of SimFeature regions
   * @return HashMap of genemodels pointing to ArrayList of overlapping
   *     features
   */
  protected static HashMap getGmHashOfOverlappedFeat
      (int debug,
       boolean considerStrand,
       ArrayList _arrListOfGMRegions,
       ArrayList _arrListOfFeatRegions) {
    HashMap gmHashOfOverlappedFeat    = new HashMap();
    HashMap gmHashOfNonOverlappedFeat = new HashMap();

    // for all gm regions
    for (Iterator itGmReg = _arrListOfGMRegions.iterator(); itGmReg.hasNext(); )
    {
      GFMergeRegion gmReg = (GFMergeRegion) itGmReg.next();

      for (Iterator itGm = gmReg.getLocateable().iterator(); itGm.hasNext(); )
      {
        GeneModel gm = (GeneModel) itGm.next();
        ArrayList featOverlapArr =
            BasicFeatureAnalysisTools.getFeatOverlapArr(considerStrand,
                                                        gm,
                                                        _arrListOfFeatRegions);
        // put only real overlaps in hash
        if(featOverlapArr.size() > 0)
        {
          gmHashOfOverlappedFeat.put(gm, featOverlapArr);
        }
        // non overlapped genemodels
        else
        {
          gmHashOfNonOverlappedFeat.put(gm, featOverlapArr);
        }
      }
    }

    // debug 401 - HashMap of gms pointing to array of overlapping features
    Debug.debug_401(debug,
                    "BasicFeatureAnalysisTools - HashMap of gms pointing"
                    + "to array of overlapping features",
                    gmHashOfOverlappedFeat);

    return gmHashOfOverlappedFeat;
  }


  /** method which gets overlapping features for genemodel
   *
   * @param considerStrand switch for strand consideration
   * @param gm genemodel
   * @param _arrListOfFeatRegions ArrayList of SimFeature regions
   * @return ArrayList of overlapping features
   */
  private static ArrayList getFeatOverlapArr(boolean considerStrand,
                                             GeneModel gm,
                                             ArrayList _arrListOfFeatRegions) {
    ArrayList featOverlapArr = new ArrayList();
    Location gmLoc = gm.getLocation();

    // find overlapping feature regions
    for (Iterator itFeatReg = _arrListOfFeatRegions.iterator();
         itFeatReg.hasNext(); ) {
      GFMergeRegion featReg = (GFMergeRegion) itFeatReg.next();
      Location featRegLoc = featReg.getLocation();

      if (LocationTools.overlaps(gmLoc, featRegLoc)) {
        ArrayList tempArr =
            BasicFeatureAnalysisTools.findOverlappingFeatInFeatReg(considerStrand,
            gm,
            featReg);
        featOverlapArr.addAll(tempArr);
      }
    }
    return featOverlapArr;
  }

  /** method wich finds overlapping features for gm in feature region
   *
   * @param considerStrand switch for strand consideration
   * @param gm genemodel
   * @param featReg region of SimFeature objects
   * @return ArrayList of overlapping features in feature region
   */
  private static ArrayList findOverlappingFeatInFeatReg(boolean considerStrand,
      GeneModel gm,
      GFMergeRegion featReg)

  {
    ArrayList overlappingFeatInFeatReg = new ArrayList();

    for (Iterator itFeat = featReg.getLocateable().iterator(); itFeat.hasNext(); ) {
      SimFeature feature = (SimFeature) itFeat.next();
      if (BasicFeatureAnalysisTools.isFeatureOverlappingGm(considerStrand,
          gm,
          feature) == true) {
        overlappingFeatInFeatReg.add(feature);
      }
    }

    return overlappingFeatInFeatReg;
  }

  /** method wich finds out if given feature is overlapping genemodel
   *
   * @param considerStrand switch for strand consideration
   * @param gm genemodel
   * @param feature SimFeature object
   * @return is overlapping / is not overlapping (true/false)
   */
  public static boolean isFeatureOverlappingGm(boolean considerStrand,
                                                GeneModel gm,
                                                SimFeature feature) {
    boolean isFeatureOverlapping = false;
    Location gmLoc = gm.getLocation();
    Location featLoc = feature.getLocation();

    // CONSIDER STRAND
    if (considerStrand == true) {
      if (LocationTools.overlaps(gmLoc, featLoc)
          && (gm.getStrand() == feature.getStrand())) {
        isFeatureOverlapping = true;
      }
    }

    // DON'T CONSIDER STRAND
    else if (considerStrand == false) {
      if (LocationTools.overlaps(gmLoc, featLoc)) {
        isFeatureOverlapping = true;
      }
    }

    return isFeatureOverlapping;
  }

}
