/*
 * This file is part of GFMerge. Copyright (c) 2007 Genome
 * Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.psu.gfmerge.datacomp;

import org.biojava.bio.seq.*;

import uk.ac.sanger.psu.gfmerge.util.*;
import uk.ac.sanger.psu.gfmerge.util.biotools.*;

/** class is representing a geneprediction program (its prediction).
 * <p>
 * For example:
 * <pre>
 * GenePredicter gp = new GenePredicter(seqOrg, seqComp, predFileName);
 * </pre>
 * 
 * @author Sebastian R. Spiegler
 * @version 1.0
 */

public class GenePredicter {

  ///////////////////////////////////////
  // attributes

    /** name of geneprediction program
     */
    private String name;
    /** average conditional probability of geneprediction program
     */
    private double acp;


    ///////////////////////////////////////
    // constructor

    /** constructor of GenePredicter objects
     * 
     * @param seqOrg annotation
     * @param seqComp gene predictions
     * @param predFileName filename of prediction file
     */
    public  GenePredicter(Sequence seqOrg, Sequence seqComp, String predFileName)
    {
      setName(retrieveName(seqComp,predFileName));
      setAcp(calcAcp(seqOrg,seqComp));

    } // end GenePredicter


  ///////////////////////////////////////
  // operations

    /** accessor method which returns name of genepredicter.
     * genepredicter name is retrieved from method-tag in prediction file.
     * 
     * @return name of genepredicter
     */
    public String getName() {
        return name;
    } // end getName

    /** accessor method which sets name of genepredicter.
     * 
     * @param _name name of genepredicter
     */
    private void setName(String _name) {
        name = _name;
    } // end setName

    /** accessor method which returns average conditional probability of genepredicter program.
     * 
     * @return acp value
     * @see uk.ac.sanger.psu.gfmerge.util.PredAccuracy
     */
    public double getAcp() {
        return acp;
    } // end getAcp

    /** accessor method which sets average conditional probability of genepredicter program.
     * 
     * @param _acp average conditional probability of genepredicter program
     * @see uk.ac.sanger.psu.gfmerge.util.PredAccuracy
     */
    private void setAcp(double _acp) {
        acp = _acp;
    } // end setAcp


    /** method which retrieves name of genepredicter program from gene prediction file.
     * 
     * @param seq sequence which contains BioJava feature with method-tag each
     * @param fname filename of geneprediction file
     * @return genepredicter name
     */
    private String retrieveName(Sequence seq, String fname)
    {
      return BioTools.getGenePredicterFromSequence(seq,fname);
    }

    /** method which retrieves average conditional probability from PredAccuracy object. 
     * 
     * @param seqOrg annotation
     * @param seqComp gene predictions
     * @return acp value
     */
    private double calcAcp(Sequence seqOrg, Sequence seqComp)
    {
      PredAccuracy predAcc = new PredAccuracy(seqOrg,seqComp);
//      System.err.println("TP " + predAcc.getTP());
//      System.err.println("TN " + predAcc.getTN());
//      System.err.println("FP " + predAcc.getFP());
//      System.err.println("FN " + predAcc.getFN());
      return predAcc.getACP();
    }

} // end GenePredicter
