/*
 * This file is part of GFMerge. Copyright (c) 2007 Genome
 * Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.psu.gfmerge.datacomp;

import java.io.*;
import java.util.*;
import org.biojava.bio.*;
import org.biojava.bio.seq.*;
import org.biojava.bio.seq.io.*;
import org.biojava.bio.seq.impl.*;
import org.biojava.bio.symbol.*;

import uk.ac.sanger.psu.gfmerge.util.biotools.*;
import uk.ac.sanger.psu.gfmerge.*;

/** class Prediction is wrapping a BioJava sequence object.
 * <p>
 * Apart from the sequence it contains an association to the GenePredicter and references to the genemodels on the sequence.
 * 
 * For example:
 * <pre>
 * Prediction prediction = new Prediction(sequence, filename, genePredicter);
 * </pre>
 * 
 * @see uk.ac.sanger.psu.gfmerge.datacomp.GenePredicter
 * @see uk.ac.sanger.psu.gfmerge.datacomp.GeneModel
 * @version 1.0
 */
public class Prediction implements PredictionAble {

  ///////////////////////////////////////
  // attributes


    /** BioJava sequence object
     */
    private Sequence sequence;
    /** reference to gene prediction program
     */
    private GenePredicter genePredicter; // 1-to-1-association to GenePredicter
    /** name of gene prediction program (method tag in prediction file)
     */
    private String genePredicterName;
    /** ArrayList of predicted genemodels on sequence
     */
    private ArrayList geneModels = new ArrayList();


  ///////////////////////////////////////
  // operations

    /** constructor of Prediction objects
     * 
     * @param _sequence BioJava sequence object
     * @param _fname filename of prediction file
     * @param _genePredicters ArrayList of genepredicters, reference to
     *      particular genepredicter is obtained by using the 
     *     genePredicterName of prediction and training data files
     */
    public  Prediction(Sequence _sequence, String _fname, ArrayList _genePredicters)
    {
      setSequence(_sequence);
      setGenePredicterName(_sequence, _fname);
      setGenePredicter(_fname, _genePredicters);  //has to be called after setGenePredicterName()
      setGeneModels(_sequence);
    } // end Prediction

    ///////////////////////////////////////
    // operations

    /** accessor method which return BioJava sequence object
     * 
     * @return BioJava sequence object
     */
    public Sequence getSequence() { //done
        return sequence;
    } // end getSequence

    /** accessor method which returns genepredicter object
     * 
     * @return genepredicter object
     */
    public GenePredicter getGenePredicter() { //done
        return genePredicter;
    } // end getGenePredicter

    /** accessor method which returns ArrayList of genemodels on sequence
     * 
     * @return ArrayList of genemodels on sequence
     */
    public ArrayList getGeneModels() { //done
        return geneModels;
    } // end getGeneModels

    /** accessor method which returns the genepredicter name in the method tag of the prediction file 
     * 
     * @return genepredicter name
     */
    public String getGenePredicterName() { //done
      return genePredicterName;
    } // end getgenePredicterName

    /** accessor method which sets reference to BioJava sequence object
     * 
     * @param _sequence BioJava sequence object
     */

    private void setSequence(Sequence _sequence) { //done
      sequence = _sequence;
    } // end setSequence

    /** accessor method which sets reference to genepredicter object
     * 
     * @param _fname filename
     * @param _genePredicters reference to genepredicter 
     */
    private void setGenePredicter(String _fname, ArrayList _genePredicters) {
      // flag for found gene predicter object which belongs to prediction
      boolean foundFlag = false;
      for(Iterator it = _genePredicters.iterator();
          it.hasNext();)
      {
        GenePredicter gp = (GenePredicter) it.next();
        if((gp.getName()).compareTo(this.genePredicterName) == 0)
        {
          this.genePredicter = gp;
          foundFlag = true;
        }
      }
      if (foundFlag == false)
      {

	  // axk

	  // This is not an error, the sequence doesn't have any CDS predicted over it

	  // System.err.println("Error");
	  // System.err.println("Missing gene predicter for prediction " + _fname);
	  //System.err.println(this.getClass());
        // System.exit(1);
      }
    } // end setGenePredicter

    /** accessor method which sets ArrayList of genemodels on sequence
     * 
     * @param _sequence ArrayList of genemodels
     */
    private void setGeneModels(Sequence _sequence) { //done
      for(Iterator it = _sequence.features(); it.hasNext();)
      {
        StrandedFeature f = (StrandedFeature) it.next();
        geneModels.add(new GeneModel(this,f));
        //debug
//          System.out.println("size GM array " + geneModels.size());
      }
    } // end setGeneModels

    /** accessor method which sets genepredicter name
     * 
     * @param _sequence BioJava sequence object obtained from prediction file
     * @param _fname filename
     */
    private void setGenePredicterName(Sequence _sequence, String _fname) {
      genePredicterName = BioTools.getGenePredicterFromSequence(_sequence, _fname);
    } // end setGenePredicterName


} // end Prediction
